! this program calculates the Cassie-Baxter contact angle between a liquid! and a surface with 3D generic geometry based on two models:
!    a) the conventional Cassie-Baxter model and
!    b) the extended Cassie-Baxter model including the Sagging radius
! When using this code in any sence, please cite the following work: https://doi.org/10.1088/2051-672X/ab9419

!======================== START MODULE ============================
!==================================================================
module variables

implicit none

save

integer junki, lvl, case
integer, allocatable:: i(:), j(:)
real*8 thetaY, pi
real*8, allocatable:: hStar(:), RsagStar(:), fsl(:), fla(:), fsl_org(:), fla_org(:), HeightStar(:)

end module
!==================================================================
!========================= END MODULE =============================


Program Generic3D_surface

use variables

implicit none


integer ilvl, Nlvl, i1, j1, i2, j2, i3, j3
integer, allocatable:: level(:), alphaStarMax(:), alphaStarmin(:), dalphaStar(:), lStarmax(:), lStarmin(:), dlStar(:)
real*8 gammaL, gammaLp, gammaLd, rho, gammaS, gammaSp, gammaSd, &
       g, lcap, dstar, cond2RHS, &
       costhetaCB, costhetaCB0
real*8, allocatable:: l0(:), alphaMin(:), alphaMax(:), dalpha(:), lMin(:), lMax(:), dl(:), zeta(:), Height(:)
character*35 filename1



g = 9.80665d0    ! gravitational acceleration
pi = 3.14159265359d0   ! the value of pi in radians 


! Read the input file ------------------------------
Open(Unit=555,file="input.in")
Read(555,*) gammaL, gammaLp, gammaLd   ! total, polar and dispersive components of the surface tension of the wetting liquid in [N/m]
Read(555,*) rho       ! density of the wetting liquid in [kg/m^3]
Read(555,*) gammaS, gammaSp, gammaSd   ! total, polar and dispersive components of the surface energy of the wetted surface in [N/m]
Read(555,*) thetaY    ! the Young contact angle in [degrees]
Read(555,*) Nlvl        ! number of levels in the topography (multiplicity)
Read(555,*) filename1     ! name of file to output the extended Cassie-Baxter angle
Allocate ( level(Nlvl), alphaStarMax(Nlvl), alphaStarmin(Nlvl), dalphaStar(Nlvl), lStarmax(Nlvl), &
           lStarmin(Nlvl), dlStar(Nlvl), l0(Nlvl), alphaMin(Nlvl), alphaMax(Nlvl), dalpha(Nlvl), lMin(Nlvl), &
           lMax(Nlvl), dl(Nlvl), zeta(Nlvl), Height(Nlvl), i(Nlvl), j(Nlvl), hStar(Nlvl), RsagStar(Nlvl), &
           fsl(Nlvl+1), fla(Nlvl+1), fsl_org(Nlvl+1), fla_org(Nlvl+1), HeightStar(Nlvl) )
Do ilvl = 1, Nlvl
   Read(555,*) level(ilvl)              ! INTEGER that identifies the 1st-level topology: 1=pillar, 2=fiber, 3=sinusoidal, 4=3Dpillars
   Read(555,*) l0(ilvl)                                 ! LOWER_LEVEL: parameter l0 in [m]
   Read(555,*) alphaMin(ilvl), alphaMax(ilvl), dalpha(ilvl)   ! LOWER_LEVEL: Radius: the minimum and maximum values of alpha and increment in [μm]
   Read(555,*) lMin(ilvl), lMax(ilvl), dl(ilvl)             ! LOWER_LEVEL: Distance between fibers: the minimum and maximum values of l and increment in [μm]
   Read(555,*) Height(ilvl)                                 ! LOWER_LEVEL: Pillar height in [μm] (applicable only when pillars are sellected)
enddo
Close(555)
! --------------------------------------------------

! convert thetaY from degrees to radians
thetaY = (pi*thetaY/180.d0)

! convert l0 from m to μm
l0 = l0 * 1d6


! Calculate the capillary length in [m] ------------
lcap = sqrt(gammaL/(rho*g))  
write(*,*) "capillary length [m] = ", lcap
! convert capillary length to μm from m
lcap = lcap * 1d6
! --------------------------------------------------
zeta = l0/lcap
write(*,*) "zeta lower [-] = ", zeta(1),"zeta upper [-] = " , zeta(2)

! Define the dimensionless paramteres alpha_star and l_star and their increments
alphaStarMax = ceiling(alphaMax/l0); alphaStarMin = ceiling(alphaMin/l0); dalphaStar = ceiling(dalpha/l0);
lStarMax = ceiling(lMax/l0); lStarMin = ceiling(lMin/l0); dlStar = ceiling(dl/l0);
!write(*,*) alphaStarMin(1), alphaStarMax(1), dalphaStar(1)
!write(*,*) lStarMin(1), lStarMax(1), dlStar(1)


! Open the file to output data to
Open(Unit=555,file=filename1)
!Open(Unit=777,file=filename2)
Open(Unit=888,file="originalCB.out")
if(Nlvl.eq.1)then
   write(555,*) " a1 [μm]     ", "   l1 [μm]    ", " θeCB [deg] "
elseif(Nlvl.eq.2)then
   write(555,*) " a1 [μm]   ", "   l1 [μm]  ", "   a2 [μm]  ", "   l2 [μm]  "," θeCB [deg] "
elseif(Nlvl.eq.3)then
   write(555,*) " a1 [μm]  ", "   l1 [μm] ", "   a2 [μm] ", "   l2 [μm] ","   a3 [μm] ", "   l3 [μm] ", " θeCB [deg]"
endif
write(888,*) " a1 [μm]     ", "   l1 [μm]    "," θCB [deg] "



      !====================================== START WITH LOWER LEVEL CALCULATIONS ===================================================
      !==============================================================================================================================
! Start varying parameters alpha and l withing the limits defined in the input file -------------

Do i1 = alphaStarMin(1), alphaStarMax(1), dalphaStar(1)  

   i(1) = i1

   Do j1 = lStarMin(1), lstarMax(1), dlStar(1)
   
      j(1) = j1      
   
      ! calculate the dimensionless protrusion length       
      hStar(1) = zeta(1) * (dble(j(1))**2)

      lvl = 1
      if(level(lvl).eq.1)then
         HeightStar(lvl) = Height(lvl)/l0(lvl)
         call pillar()
      elseif(level(lvl).eq.2)then
         call fiber()
      elseif(level(lvl).eq.3)then
         call sinusoidal()
      elseif(level(lvl).eq.4)then
         HeightStar(lvl) = Height(lvl)/l0(lvl)
         call pillar()
         call pillar3D()
      else
         write(*,*) "Level", lvl, " topology not properly defined"
         stop
      endif

      if(Nlvl.eq.1)then ! we only have one level of topography, so write the results              
         ! Calculate the contact angle based on the extended Cassie-Baxter model 
         costhetaCB = fsl(lvl)*cos(thetaY) - fla(lvl)

         ! Calculate the contact angle based on the original Cassie-Baxter model
         costhetaCB0 = fsl_org(lvl)*cos(thetaY) - fla_org(lvl)

         write(555,30) dble(i(lvl))*l0(lvl), dble(j(lvl))*l0(lvl), acos(costhetaCB)*180.d0/pi, case
         write(888,10) dble(i(lvl))*l0(lvl), dble(j(lvl))*l0(lvl), acos(costhetaCB0)*180.d0/pi 

      elseif(Nlvl.gt.1)then ! 2nd level of topography
         ! Transfer the 1st level values to the accumulative variable
         fsl(Nlvl+1) = fsl(lvl)
         fla(Nlvl+1) = fla(lvl)

         ! Start varying parameters alpha and l withing the limits defined in the input file -------------
         Do i2 = alphaStarMin(2), alphaStarMax(2), dalphaStar(2)

            i(2) = i2

            Do j2 = lStarMin(2), lstarMax(2), dlStar(2)

               j(2) = j2

               ! calculate the dimensionless protrusion length       
               hStar(2) = zeta(2) * (dble(j(2))**2)

               lvl = 2
               if(level(lvl).eq.1)then
                  HeightStar(lvl) = Height(lvl)/l0(lvl)
                  call pillar()
               elseif(level(lvl).eq.2)then
                  call fiber()
               elseif(level(lvl).eq.3)then
                  call sinusoidal()
               else
                  write(*,*) "Level", lvl, " topology not properly defined"
                  stop
               endif

               ! Compute the fsl and fla for the two-scale topography
               fsl(Nlvl+1) = fsl(lvl-1)*fsl(lvl)
               fla(Nlvl+1) = fla(lvl-1) + fla(lvl)*fsl(lvl-1)

               if(Nlvl.eq.2)then ! we have two levels of topography, so write the results
                  ! Calculate the contact angle based on the extended Cassie-Baxter model 
                  costhetaCB = fsl(Nlvl+1)*cos(thetaY) - fla(Nlvl+1)

                  write(555,40) dble(i(lvl-1))*l0(lvl-1), dble(j(lvl-1))*l0(lvl-1), dble(i(lvl))*l0(lvl), &
                                dble(j(lvl))*l0(lvl), acos(costhetaCB)*180.d0/pi, case
               elseif(Nlvl.eq.3)then ! there is a third level of topography
                  ! Start varying parameters alpha and l withing the limits defined in the input file -------------
                  Do i3 = alphaStarMin(3), alphaStarMax(3), dalphaStar(3)

                     i(3) = i3

                     Do j3 = lStarMin(3), lstarMax(3), dlStar(3)

                        j(3) = j3

                        ! calculate the dimensionless protrusion length       
                        hStar(3) = zeta(3) * (dble(j(3))**2)

                        lvl = 3
                        if(level(lvl).eq.1)then
                           HeightStar(lvl) = Height(lvl)/l0(lvl)
                           call pillar()
                        elseif(level(lvl).eq.2)then
                           call fiber()
                        elseif(level(lvl).eq.3)then
                           call sinusoidal()
                        else
                           write(*,*) "Level", lvl, " topology not properly defined"
                           stop
                        endif

                        ! Compute the fsl and fla for the three-scale topography
                        fsl(Nlvl+1) = fsl(lvl-2)*fsl(lvl-1)*fsl(lvl)
                        fla(Nlvl+1) = fla(lvl-2) + fla(lvl-1)*fsl(lvl-2) + fla(lvl)*fsl(lvl-1)

                        ! Calculate the contact angle based on the extended Cassie-Baxter model 
                        costhetaCB = fsl(Nlvl+1)*cos(thetaY) - fla(Nlvl+1)

                        write(555,50) dble(i(lvl-2))*l0(lvl-2), dble(j(lvl-2))*l0(lvl-2), &
                                      dble(i(lvl-1))*l0(lvl-1), dble(j(lvl-1))*l0(lvl-1), &
                                      dble(i(lvl))*l0(lvl), dble(j(lvl))*l0(lvl), &
                                      acos(costhetaCB)*180.d0/pi, case

                     enddo !j3

                  enddo !i3

               endif

            enddo !j2

         enddo !i2

      endif

   enddo !j1

enddo !i1
Close(555)


10 Format(3(G12.5,2x))
20 Format(5(G12.5,2x))
30 Format(3(G12.5,2x),I5)
40 Format(5(G12.5,2x),I5)
50 Format(7(G12.5,2x),I5)


end
